@** Main Program.

Here is a quick skeleton that I put together to show how the
inverse adding-doubling code works.  I have only cursorily tested
this.  If you find obvious bugs, they are probably real but should
not extend beyond this code snippet. 

I create an empty file \.{iad\_main.h} to simplify the Makefile
@(iad_main.h@>=

@ All the actual output for this web file goes into \.{iad\_main.c}

@(iad_main.c@>=

@<Include files for |main|@>@;

@<print version function@>@;
@<print usage function@>@;
@<stringdup together function@>@;
@<seconds elapsed function@>@;
@<print error legend@>@;
@<print dot function@>@;
@<calculate coefficients function@>@;
@<parse string into array function@>@;
 
int main (int argc, char **argv)
{
    @<Declare variables for |main|@>@;
    
    @<Handle options@>@;
        
    Initialize_Measure(&m);
	r.method.quad_pts = 8;  
    @<Command-line changes to |m|@>@;
    
    if (process_command_line) {
		@<Count command-line measurements@>@;
		@<Calculate and write optical properties@>@;
        return 0;
    }

    @<prepare file for reading@>@;
    if (Read_Header (stdin, &m, &params) == 0) {
        start_time = clock();
        while (Read_Data_Line (stdin, &m, params) == 0) {
            @<Calculate and write optical properties@>@;
            first_line = 0;
        }
    }
    if (cl_verbosity>0) fprintf(stderr,"\n\n");
    if (cl_verbosity>1) print_error_legend();
    return 0;
}

@ The first two defines are to stop Visual C++ from silly complaints
@<Include files for |main|@>=
#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_WARNINGS

#define NO_SLIDES            0
#define ONE_SLIDE_ON_TOP     1
#define TWO_IDENTICAL_SLIDES 2
#define ONE_SLIDE_ON_BOTTOM  3

#define MR_IS_ONLY_RD        1
#define MT_IS_ONLY_TD        2
#define NO_UNSCATTERED_LIGHT 3

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "ad_globl.h"
#include "ad_prime.h"
#include "iad_type.h"
#include "iad_pub.h"
#include "iad_io.h"
#include "iad_calc.h"
#include "iad_util.h"
#include "mygetopt.h"
#include "version.h"
#include "mc_lost.h"
#include "ad_frsnl.h"

extern char *optarg;
extern int   optind;

@ @<Declare variables for |main|@>=
  struct measure_type m;
  struct invert_type r;
  char *g_out_name = NULL;
  char c;
  int first_line = 1;

  long n_photons = 100000;	
  int MC_iterations = 20;
  int any_error = 0;
  int process_command_line = 0;
  int params = 0;
  
  int    cl_quadrature_points = 8;
  int    cl_verbosity = 2;
  
  double cl_default_a   = UNINITIALIZED;
  double cl_default_g   = UNINITIALIZED;
  double cl_default_b   = UNINITIALIZED;
  double cl_default_mua = UNINITIALIZED;
  double cl_default_mus = UNINITIALIZED;
  double cl_tolerance   = UNINITIALIZED;
  double cl_no_unscat   = UNINITIALIZED;

  double cl_beam_d      = UNINITIALIZED;
  double cl_sample_d    = UNINITIALIZED;
  double cl_sample_n    = UNINITIALIZED;
  double cl_slide_d     = UNINITIALIZED;
  double cl_slide_n     = UNINITIALIZED;
  double cl_slides      = UNINITIALIZED;

  double cl_UR1         = UNINITIALIZED;     
  double cl_UT1         = UNINITIALIZED;
  double cl_Tc          = UNINITIALIZED;
  
  double cl_num_spheres = UNINITIALIZED;
  double cl_sphere_one[5] = {UNINITIALIZED, UNINITIALIZED, UNINITIALIZED,
                             UNINITIALIZED, UNINITIALIZED };
  double cl_sphere_two[5] = {UNINITIALIZED, UNINITIALIZED, UNINITIALIZED,
                             UNINITIALIZED, UNINITIALIZED };

  clock_t start_time=clock();

@ use the |mygetop| to process options.  We only handle help at the moment

@<Handle options@>=
    while ((c = my_getopt(argc, argv, "?1:2:a:A:b:B:c:d:D:e:F:g:G:hn:N:M:o:p:q:r:S:t:u:vV:x:")) 
            != EOF) {
        switch (c) {

            case '1':
            	parse_string_into_array(optarg, cl_sphere_one, 5);
                break;

            case '2':
            	parse_string_into_array(optarg, cl_sphere_two, 5);
                break;

            case 'a':
                cl_default_a = strtod(optarg, NULL);
                break;

            case 'A':
                cl_default_mua = strtod(optarg, NULL);
                break;
                
            case 'b':
                cl_default_b = strtod(optarg, NULL);
                break;

            case 'B':
                cl_beam_d = strtod(optarg, NULL);
                break;

            case 'c':
                cl_no_unscat = strtod(optarg, NULL);
                break;

            case 'd':
                cl_sample_d = strtod(optarg, NULL);
                break;

            case 'D':
                cl_slide_d = strtod(optarg, NULL);
                break;

            case 'e':
                cl_tolerance = strtod(optarg, NULL);
                break;

            case 'F':
                cl_default_mus = strtod(optarg, NULL);
                break;

            case 'g':
                cl_default_g = strtod(optarg, NULL);
                break;
                
            case 'G':
            	if (optarg[0]=='0')
            		cl_slides = NO_SLIDES;
            	else if (optarg[0]=='2')
            		cl_slides = TWO_IDENTICAL_SLIDES;
            	else if (optarg[0]=='t' || optarg[0]=='T')
            		cl_slides = ONE_SLIDE_ON_TOP;
            	else if (optarg[0]=='b' || optarg[0]=='B')
            		cl_slides = ONE_SLIDE_ON_BOTTOM;
                else {
                    fprintf(stderr, "Argument for -G option must be 't' (for top)");
                    fprintf(stderr, " or 'b' for bottom or '0' or '2'\n");
                    exit(1);
                }
                break;
                    
            case 'M':
                MC_iterations = (int) strtod(optarg, NULL);
                break;

            case 'n':
                cl_sample_n = strtod(optarg, NULL);
                break;

            case 'N':
                cl_slide_n = strtod(optarg, NULL);
                break;

            case 'o':
                g_out_name = strdup(optarg);
                break;
    
            case 'p':
                n_photons = (int) strtod(optarg, NULL);
                break;

            case 'q':
                cl_quadrature_points = (int) strtod(optarg, NULL);
                if (cl_quadrature_points % 4 != 0) {
                    fprintf(stderr, "Number of quadrature points must be a multiple of 4\n");
                    exit(1);
                }
                break;

            case 'r':
                cl_UR1 = strtod(optarg, NULL);
                process_command_line = 1;
                break;

            case 'S':
                cl_num_spheres = (int) strtod(optarg, NULL);
                break;

            case 't':
                cl_UT1 = strtod(optarg, NULL);
                process_command_line=1;
                break;

            case 'u':
                cl_Tc = strtod(optarg, NULL);
                process_command_line=1;
                break;

            case 'v':
                print_version();
                break;
                
            case 'V':
                cl_verbosity = strtod(optarg, NULL);
                break;

            case 'x':
                Set_Debugging( (int) strtod(optarg, NULL));
                break;

            default:
            case 'h':
            case '?':
                print_usage();
                break;
        }
    }
    
    argc -= optind;
    argv += optind;

@ Make sure that the file is not named '-' and warn about too many files

@<prepare file for reading@>=
    if (argc > 1) {
        fprintf(stderr, "Only a single file can be processed at a time\n");
        fprintf(stderr, "try 'apply iad file1 file2 ... fileN'\n");
        exit(1);
    }

    if (argc == 1 && strcmp(argv[0],"-")!=0) {  /* filename exists and != "-" */
		int n;
		char *base_name, *rt_name;
		base_name = strdup(argv[0]);
		n = (int) (strlen(base_name) - strlen(".rxt"));
		
		if (n>0 && strstr(base_name+n,".rxt") != NULL) 
			base_name[n]='\0';

		rt_name = strdup_together(base_name,".rxt");
		
        if (freopen(argv[0],"r",stdin)==NULL &&
            freopen(rt_name,"r",stdin)==NULL) {
            fprintf(stderr, "Could not open either '%s' or '%s'\n", 
                            argv[0], rt_name);
            exit(1);
        }

        if (g_out_name==NULL) 
            g_out_name=strdup_together(base_name,".txt");
   
        free(rt_name);
        free(base_name);
    }
    
    if (g_out_name!=NULL) {
        if (freopen(g_out_name,"w",stdout)==NULL) {
            fprintf(stderr, "Could not open file '%s' for output\n", g_out_name);
            exit(1);
        }
    }


@ Need to explicitly reset |r.search| each time through the loop,
because it will get altered by the calculation process.  We want to be
able to let different lines have different constraints.  In particular
consider the file |newton.tst|.  In that file the first two rows contain
three real measurements and the last two have the collimated transmission
explicitly set to zero --- in other words there are really only two 
measurements.

@<Calculate and write optical properties@>=
{
	@<Local Variables for Calculation@>@;

	Initialize_Result(m, &r);
    @<Command-line changes to |r|@>@;

	r.search = determine_search(m,r);
	
	@<Make copies of |m| and |r| for MC@>@;

	@<Write Header @>@;
	Inverse_RT (m, &r);
	calculate_coefficients(m,r,&LR,&LT,&mu_sp_sphere,&mu_a_sphere);

	@<Improve result using Monte Carlo@>@;
		
	@<Write Result@>@;
	r.error = IAD_NO_ERROR;
}  


@ 	@<Local Variables for Calculation@>=
    static int rt_data_count = 0;
    static int mc_iter_count = 0;
	struct measure_type m_mc, m_none;
	struct invert_type r_mc, r_none;
    int mc_iter = 0;
    
    double ur1=0;
    double ut1=0;
    double uru=0;
    double utu=0;
    double ur1_lost=0;
    double ut1_lost=0;
    double uru_lost=0;
    double utu_lost=0;
    double mu_a_none=0;
    double mu_a_sphere=0;
    double mu_a_both=0;
    double mu_a_mc=0;
    double mu_sp_none=0;
    double mu_sp_sphere=0;
    double mu_sp_both=0;
    double mu_sp_mc=0;
    double LR=0;
    double LT=0;
    
    rt_data_count++;

@ @<Command-line changes to |r|@>=  
    
    r.method.quad_pts = cl_quadrature_points;  

    if (cl_default_a != UNINITIALIZED) {
    	r.default_a = cl_default_a;
    }
    
    if (cl_default_mua != UNINITIALIZED) {
    	r.default_mua = cl_default_mua;
    	if (cl_sample_d != UNINITIALIZED) 
    		r.default_ba = cl_default_mua * cl_sample_d;
    	else
    		r.default_ba = cl_default_mua * m.slab_thickness;
    }
    
    if (cl_default_mus != UNINITIALIZED) {
    	r.default_mus = cl_default_mus;
    	if (cl_sample_d != UNINITIALIZED) 
    		r.default_bs = cl_default_mus * cl_sample_d;
    	else
    		r.default_bs = cl_default_mus * m.slab_thickness;
    }
    
    if (cl_default_b != UNINITIALIZED) {
    	r.default_b = cl_default_b;
    }
    
    if (cl_default_g != UNINITIALIZED) {
    	r.default_g = cl_default_g;
    }
    
    if (cl_tolerance != UNINITIALIZED) {
    	r.tolerance = cl_tolerance;
    	r.MC_tolerance = cl_tolerance;
    }


@ @<Write Header @>=

if (rt_data_count==1 && cl_verbosity>0) {
	Write_Header (m, r, params);
	if (MC_iterations > 0) {
		if (n_photons>=0)
		   fprintf(stdout,"#  Photons used to estimate lost light =   %ld\n",n_photons);
		else
		   fprintf(stdout,"#     Time used to estimate lost light =   %ld ms\n",-n_photons);
	} else
		fprintf(stdout,"#  Photons used to estimate lost light =   0\n");

	fprintf(stdout,"#\n");
	if (Debug(DEBUG_LOST_LIGHT)) {
		fprintf(stdout,"#      \tMeas\tCalc \t");
		fprintf(stdout,"Meas \tCalc \tBest \tBest \tBest \t");
		fprintf(stdout,"Lost \tLost \tLost \tLost \tNone \tSpOnly\tMConly\t");
		fprintf(stdout,"None \tSpOnly\tMConly\tMC\tAD\tError\n");
		fprintf(stdout,"##wave\tM_R \tM_R  \t");
		fprintf(stdout,"M_T \tM_T  \tmu_a\tmu_s'\t  g\tR_drct\tR_diff\tT_drct\tT_diff\t");
		fprintf(stdout,"mu_a\tmu_a\tmu_a\tmu_s'\tmu_s'\tmu_s'\titer\titer\tNumber\n");
		fprintf(stdout,"# [nm] \t[---]\t[---]\t[---]\t[---]\t");
		fprintf(stdout,"[1/mm]\t[1/mm]\t[---]\t[---]\t[---]\t[---]\t");
		fprintf(stdout,"[---]\t[1/mm]\t[1/mm]\t");
		fprintf(stdout,"[1/mm]\t[1/mm]\t[1/mm]\t[1/mm]\t[---]\t[---]\t[---]\n");
	} else {
		fprintf(stdout,"#      \tMeas\tCalc\t");
		fprintf(stdout,"Meas\tCalc\t");
		fprintf(stdout,"Calc\tCalc\tCalc\tError\n");
		fprintf(stdout,"##wave\tM_R \tM_R \tM_T \tM_T \t");
		fprintf(stdout,"mu_a\tmu_s'\tg\tMC it.\tNumber\n");
		fprintf(stdout,"# [nm] \t[---]\t[---]\t[---]\t[---]\t");
		fprintf(stdout,"[1/mm]\t[1/mm]\t[---]\t[---]\t[---]\n");
	}

}

@ Just to make sure that all the proper fields are zero.  Once |m| (and |r|)
have been set up, then copy these into |m_mc| and |m_none| so those are
initialized properly.

@<Make copies of |m| and |r| for MC@>=
    m.ur1_lost = 0;
    m.ut1_lost = 0;
    m.uru_lost = 0;
    m.utu_lost = 0;
    r.error = IAD_NO_ERROR;
    m.f_r = 0.0;
 
 	memcpy(&m_none, &m, sizeof(struct measure_type));
  	memcpy(&r_none, &r, sizeof(struct invert_type));
 	memcpy(&m_mc,   &m, sizeof(struct measure_type));
  	memcpy(&r_mc,   &r, sizeof(struct invert_type));

	
@ Use Monte Carlo to figure out how much light leaks out.  We use
the sphere corrected values as the starting values and only do try
Monte Carlo when spheres are used, the albedo unknown or non-zero, and
there has been no error.  The sphere parameters must be known because
otherwise the beam size and the port size are unknown.

@<Improve result using Monte Carlo@>=

@<initial values for scattering and absorption@>@;

if (m.num_spheres > 0 && r.default_a != 0 && r.error == IAD_NO_ERROR) {

	@<Result for no sphere corrections and no light loss@>@;
	
	while (mc_iter < MC_iterations && r.error == IAD_NO_ERROR) {
		double mu_sp_last   = mu_sp_both;
		double mu_a_last    = mu_a_both;
		double ur1_max_loss = 0;
		double ut1_max_loss = 0;

		MC_Lost(m, r, -1000, &ur1, &ut1, &uru, &utu, 
								 &ur1_lost, &ut1_lost, &uru_lost, &utu_lost);   
		mc_iter_count++;
		mc_iter++;
									
		@<Subtract Lost Light From Measurements@>
			
		Inverse_RT (m, &r);
		calculate_coefficients(m,r,&LR,&LT,&mu_sp_both,&mu_a_both);

		@<Result for no sphere corrections but including light loss@>@;
		
		if (fabs(mu_a_last -mu_a_both )/(mu_a_both +0.0001) < r.MC_tolerance &&
			fabs(mu_sp_last-mu_sp_both)/(mu_sp_both+0.0001) < r.MC_tolerance) break;

		@<Print intermediate lost light results@>@;
	}
}

@ @<initial values for scattering and absorption@> =
	mu_sp_both   = mu_sp_sphere;
	mu_sp_none   = mu_sp_both;
	mu_sp_mc     = mu_sp_both;
	mu_a_both    = mu_a_sphere;
	mu_a_mc      = mu_a_both;
	mu_a_none    = mu_a_both;	

@ This is the worst possible estimate.  It is only needed if we are reporting
light loss estimates.  This is the case that corresponds to no sphere corrections
and no light loss corrections.

@<Result for no sphere corrections and no light loss@>=

	if (Debug(DEBUG_LOST_LIGHT)) {
		m_none.num_spheres = 0;
		Inverse_RT(m_none, &r_none);
		calculate_coefficients(m_none,r_none,&LR,&LT,&mu_sp_none,&mu_a_none);
	}

@ Correct for light loss corrections.  The idea is that from one Monte Carlo
simulation to the next, the actual amount of lost light may change, but the
relative amount remains pretty constant.  By calculating the ratio of lost
light to the diffusely scattered reflectance or transmittance, then we have
a much better correction.  |FRACTION| is used here and in |Calculate_Distance|
to turn this effect on and off.

@<Subtract Lost Light From Measurements@>=

	m.ur1_lost = ur1_lost;
	m.ut1_lost = ut1_lost;
	m.uru_lost = uru_lost;
	m.utu_lost = utu_lost;
	if (FRACTION) {
		double Rc, Tc;
		Sp_mu_RT(r.slab.n_top_slide, r.slab.n_slab,r.slab.n_bottom_slide, 
				 r.slab.b_top_slide, r.slab.b, r.slab.b_bottom_slide, 1.0, &Rc, &Tc);
		if ( ur1-Rc > 0.01) m.ur1_lost = ur1_lost/(ur1-Rc);
		if ( ut1-Tc > 0.01) m.ut1_lost = ut1_lost/(ut1-Tc);
	}

	if (0) {
		Max_Light_Loss(m, r, &ur1_max_loss, &ut1_max_loss);
		m.ur1_lost = ur1_max_loss;
		m.ut1_lost = ut1_max_loss;
	}
	
	if (0 && ur1_lost+m.m_r+ut1_lost+m.m_t >1) {
		struct AD_slab_type s;
		double ad_ur1, ad_ut1, ad_uru, ad_utu;

		s.a = r.a;
		s.b = r.b;
		s.g = r.g;
		s.phase_function = HENYEY_GREENSTEIN;
		s.n_slab = m.slab_index;
		s.n_top_slide = m.slab_top_slide_index;
		s.n_bottom_slide= m.slab_top_slide_index;
		s.b_top_slide = 0;
		s.b_bottom_slide = 0;

		RT(32, &s, &ad_ur1, &ad_ut1, &ad_uru, &ad_utu);
		fprintf(stderr, "UR1_AD=%7.5f UR1_MC=%7.5f ", ad_ur1, ur1); 
		fprintf(stderr, "UR1_LOST=%7.5f UR1_MR=%7.5f\n", ur1_lost,m.m_r); 
		fprintf(stderr, "UT1_AD=%7.5f UT1_MC=%7.5f ", ad_ut1, ut1); 
		fprintf(stderr, "UT1_LOST=%7.5f UT1_MT=%7.5f\n", ut1_lost, m.m_t); 
		r.error = IAD_EXCESSIVE_LIGHT_LOSS;
		break;
	}

@ no sphere corrections, but MC light loss corrections.  This is only
needed if we are debugging the lost light.

@<Result for no sphere corrections but including light loss@>=

	if (Debug(DEBUG_LOST_LIGHT)) {
		m_mc.num_spheres = 0;
		m_mc.ur1_lost    = ur1_lost;
		m_mc.ut1_lost    = ut1_lost;
		m_mc.uru_lost    = uru_lost;
		m_mc.utu_lost    = utu_lost;
		Inverse_RT (m_mc, &r_mc);
		calculate_coefficients(m_mc,r_mc,&LR,&LT,&mu_sp_mc,&mu_a_mc);
	}

@ When debugging lost light, it is handy to see how each iteration changes
the calculated values for the optical properties.  We do that here if we are
debugging, otherwise we just print a number or something to keep the user from
wondering what is going on.

@<Print intermediate lost light results@>=
	if (Debug(DEBUG_LOST_LIGHT)) {
		if (m.lambda != 0)
			fprintf(stderr, "%6.1f\t", m.lambda);
		else 
			fprintf(stderr, "      \t");
		  
		fprintf(stderr, "%6.4f\t%6.4f\t", m.m_r, LR);	
		fprintf(stderr, "%6.4f\t%6.4f\t", m.m_t, LT);
		fprintf(stderr, "%6.4f\t", mu_a_both);
		fprintf(stderr, "%6.4f\t", mu_sp_both);
		fprintf(stderr, "%6.4f\t", r.g);
		
		fprintf(stderr, "%6.4f\t%6.4f\t", m.ur1_lost, m.uru_lost);
		fprintf(stderr, "%6.4f\t%6.4f\t", m.ut1_lost, m.utu_lost);
		fprintf(stderr, "%6.4f\t", mu_a_none);
		fprintf(stderr, "%6.4f\t", mu_a_mc);
		fprintf(stderr, "%6.4f\t", mu_a_sphere);
	
		fprintf(stderr, "%6.4f\t", mu_sp_none);
		fprintf(stderr, "%6.4f\t", mu_sp_mc);
		fprintf(stderr, "%6.4f\t", mu_sp_sphere);
		fprintf(stderr, "%2d\t", mc_iter);
		fprintf(stderr, "%3d\t", r.iterations);
		if (FRACTION) {
			fprintf(stderr, "%4.1f\t", 100.0*m.ur1_lost);
			fprintf(stderr, "%4.1f\t", 100.0*m.ut1_lost);
		} else {
			fprintf(stderr, "%4.1f\t", 100.0*m.ur1_lost/(ur1+0.001));
			fprintf(stderr, "%4.1f\t", 100.0*m.ut1_lost/(ut1+0.001));
		}
		
		fprintf(stderr, "%2d\n",r.error);
	} else {
		if (cl_verbosity > 0) 
			print_dot(start_time, r.error, mc_iter_count, 
					  rt_data_count, mc_iter, &any_error);
	}

@ @<Write Result@>=
	if (m.lambda != 0)
		fprintf(stdout, "%6.1f", m.lambda);
	else 
		fprintf(stdout, "      ");
		
	fprintf(stdout, "\t");
	  
	fprintf(stdout, "%6.4f\t%6.4f\t", m.m_r, LR);	
	fprintf(stdout, "%6.4f\t%6.4f\t", m.m_t, LT);
	fprintf(stdout, "%6.4f\t", mu_a_both);
	fprintf(stdout, "%6.4f\t", mu_sp_both);
	fprintf(stdout, "%6.4f\t", r.g);
	
	if (Debug(DEBUG_LOST_LIGHT)) {
		fprintf(stdout, "%6.4f\t%6.4f\t", m.ur1_lost, m.uru_lost);
		fprintf(stdout, "%6.4f\t%6.4f\t", m.ut1_lost, m.utu_lost);
		fprintf(stdout, "%6.4f\t", mu_a_none);
		fprintf(stdout, "%6.4f\t", mu_a_mc);
		fprintf(stdout, "%6.4f\t", mu_a_sphere);
	
		fprintf(stdout, "%6.4f\t", mu_sp_none);
		fprintf(stdout, "%6.4f\t", mu_sp_mc);
		fprintf(stdout, "%6.4f\t", mu_sp_sphere);
		fprintf(stdout, "%2d\t", mc_iter);
		fprintf(stdout, "%3d\t", r.iterations);
	}
	
	fprintf(stdout, "%2d\n", r.error);
	fflush(stdout);
	   
				
	if (Debug(DEBUG_LOST_LIGHT)) 
		fprintf(stderr,"\n");
	else {
		if (cl_verbosity > 0) 
			print_dot(start_time, r.error, mc_iter_count, rt_data_count, 99, &any_error);
	}

@ @<Testing MC code@>=
{
    struct AD_slab_type s;
    double ur1, ut1, uru, utu;
    double adur1, adut1, aduru, adutu;
    s.a = 0.0;
    s.b = 0.5;
    s.g = 0.0;
    s.phase_function = HENYEY_GREENSTEIN;
    s.n_slab = 1.0;
    s.n_top_slide = 1.0;
    s.n_bottom_slide = 1.0;
    s.b_top_slide = 0;
    s.b_bottom_slide = 0;
    
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);

    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);

    fprintf(stderr, "   UR1            UT1            URU              UTU\n");
    fprintf(stderr, "  AD   MC        AD   MC        AD   MC          AD   MC \n");
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);

    s.b = 100.0;
    s.n_slab = 1.5;
    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);

    s.n_slab = 2.0;
    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);

    s.n_slab = 1.5;
    s.n_top_slide = 1.5;
    s.n_bottom_slide = 1.5;
    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);

    s.n_slab = 1.3;
    s.n_top_slide = 1.5;
    s.n_bottom_slide = 1.5;
    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);

    s.a = 0.5;
    s.b = 1.0;
    s.n_slab = 1.0;
    s.n_top_slide = 1.0;
    s.n_bottom_slide = 1.0;
    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);

    s.g = 0.5;
    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);

    s.n_slab = 1.5;
    fprintf(stderr, "\na=%5.4f b=%5.4f g=%5.4f n=%5.4f ns=%5.4f\n", 
           s.a, s.b, s.g, s.n_slab, s.n_top_slide);
    MC_RT(    s,&ur1,  &ut1,  &uru,  &utu);   
    RT   (32,&s,&adur1,&adut1,&aduru,&adutu);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f   ", adur1, ur1, adut1, ut1);
    fprintf(stderr, "%5.4f %5.4f   %5.4f %5.4f\n ", aduru, uru, adutu, utu);
}
@ @<old formatting@>=

    if (cl_verbosity > 0 && count % 100 == 0) fprintf(stderr,"\n");
    if (cl_verbosity > 0) 
        printf (format2, m.m_r, m.m_t, m.m_u, r.a, r.b, r.g, r.final_distance);
    else
        printf("%9.5f\t%9.5f\t%9.5f\t%9.5f\n",r.a,r.b,r.g,r.final_distance);


@ Stuff the command line arguments that should be constant over the entire
inversion process into the measurement record  and
set up the result record to handle the arguments properly so that the optical
properties can be determined.  

@<Command-line changes to |m|@>=
 	if (cl_sample_n != UNINITIALIZED) 
   		m.slab_index = cl_sample_n;

 	if (cl_slide_n != UNINITIALIZED) {
    	m.slab_bottom_slide_index = cl_slide_n;
    	m.slab_top_slide_index    = cl_slide_n;
	}
	
 	if (cl_sample_d != UNINITIALIZED) 
   		m.slab_thickness = cl_sample_d;

 	if (cl_beam_d != UNINITIALIZED) 
   		m.d_beam = cl_beam_d;

 	if (cl_slide_d != UNINITIALIZED) {
    	m.slab_bottom_slide_thickness = cl_slide_d;
    	m.slab_top_slide_thickness    = cl_slide_d;
	}
  	
 	if (cl_slides == NO_SLIDES) {
    	m.slab_bottom_slide_index     = 1.0;
    	m.slab_bottom_slide_thickness = 0.0;
    	m.slab_top_slide_index        = 1.0;
    	m.slab_top_slide_thickness    = 0.0;
	}

 	if (cl_slides == ONE_SLIDE_ON_TOP) {
    	m.slab_bottom_slide_index     = 1.0;
    	m.slab_bottom_slide_thickness = 0.0;
	}

 	if (cl_slides == ONE_SLIDE_ON_BOTTOM) {
    	m.slab_top_slide_index        = 1.0;
    	m.slab_top_slide_thickness    = 0.0;
	}

    if (cl_num_spheres != UNINITIALIZED)
    	m.num_spheres = (int) cl_num_spheres;
    
    if (cl_sphere_one[4] != UNINITIALIZED) {
		double d_sample_r, d_entrance_r, d_detector_r;
		
		m.d_sphere_r     = cl_sphere_one[0];
		d_sample_r       = cl_sphere_one[1];
		d_entrance_r     = cl_sphere_one[2];
		d_detector_r     = cl_sphere_one[3];
		m.rw_r           = cl_sphere_one[4];
		
		m.as_r = (d_sample_r   / m.d_sphere_r) * (d_sample_r   / m.d_sphere_r);
		m.ae_r = (d_entrance_r / m.d_sphere_r) * (d_entrance_r / m.d_sphere_r);
		m.ad_r = (d_detector_r / m.d_sphere_r) * (d_detector_r / m.d_sphere_r);
		m.aw_r = 1.0 - m.as_r - m.ae_r - m.ad_r;

		m.d_sphere_t = m.d_sphere_r;
		m.as_t       = m.as_r;
		m.ae_t       = m.ae_r;
		m.ad_t       = m.ad_r;
		m.aw_t       = m.aw_r;
		m.rw_t       = m.rw_r;
		
		if (cl_num_spheres == UNINITIALIZED)
    		m.num_spheres = 1;
	}
	
    if (cl_sphere_two[4] != UNINITIALIZED) {
		double d_sample_t, d_entrance_t, d_detector_t;
		
		m.d_sphere_t     = cl_sphere_two[0];
		d_sample_t       = cl_sphere_two[1];
		d_entrance_t     = cl_sphere_two[2];
		d_detector_t     = cl_sphere_two[3];
		m.rw_t           = cl_sphere_two[4];
		
		m.as_t = (d_sample_t   / m.d_sphere_t) * (d_sample_t   / m.d_sphere_t);
		m.ae_t = (d_entrance_t / m.d_sphere_t) * (d_entrance_t / m.d_sphere_t);
		m.ad_t = (d_detector_t / m.d_sphere_t) * (d_detector_t / m.d_sphere_t);
		m.aw_t = 1.0 - m.as_t - m.ae_t - m.ad_t;

		if (cl_num_spheres == UNINITIALIZED)
    		m.num_spheres = 2;
	}

    if ((cl_no_unscat == MR_IS_ONLY_RD) || (cl_no_unscat == NO_UNSCATTERED_LIGHT))
    	m.sphere_with_rc = 0.0;

    if ((cl_no_unscat == MT_IS_ONLY_TD) || (cl_no_unscat == NO_UNSCATTERED_LIGHT))
    	m.sphere_with_tc = 0.0;
 
 	if (cl_UR1 != UNINITIALIZED) 
		m.m_r = cl_UR1;

	if (cl_UT1 != UNINITIALIZED) 
		m.m_t = cl_UT1;

	if (cl_Tc != UNINITIALIZED) 
		m.m_u = cl_Tc;

@ put the values for command line reflection and transmission into the measurement record.

@<Count command-line measurements@>=
    
	m.num_measures=3;
	if (m.m_t == 0) m.num_measures--;
	if (m.m_u == 0) m.num_measures--;
	params = m.num_measures;
	
	if (m.num_measures==3) {
		/* need to fill slab entries to calculate the optical thickness */
		struct AD_slab_type s;
		s.n_slab=m.slab_index;        
		s.n_top_slide=m.slab_top_slide_index; 
		s.n_bottom_slide=m.slab_bottom_slide_index; 
		s.b_top_slide=0;
		s.b_bottom_slide=0;
		cl_default_b = What_Is_B(s, m.m_u);
	 }
	 
@ @<print version function@>=

static void print_version(void)
{
	fprintf(stderr, "iad %s\n",Version);
	fprintf(stderr, "Copyright 2009 Scott Prahl, prahl@@bme.ogi.edu\n");
	fprintf(stderr, "          (see Applied Optics, 32:559-568, 1993)\n");
	exit(0);
}

@ @<print usage function@>=
static void print_usage(void)
{
	fprintf(stderr, "iad %s\n\n",Version);
	fprintf(stderr, "iad finds optical properties from measurements\n\n");
	fprintf(stderr, "Usage:  iad [options] input\n\n");
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "  -1 '# # # # #'   reflection sphere parameters \n");
	fprintf(stderr, "  -2 '# # # # #'   transmission sphere parameters \n");
	fprintf(stderr, "     'sphere d, sample d, entrance d, detector d, wall r'\n");
	fprintf(stderr, "  -a #             use this albedo \n");
	fprintf(stderr, "  -A #             use this absorption coefficient \n");
	fprintf(stderr, "  -b #             use this optical thickness \n");
	fprintf(stderr, "  -B #             beam diameter \n");
	fprintf(stderr, "  -c #             measurements have unscattered light?\n");
	fprintf(stderr, "  -d #             thickness of sample \n");
	fprintf(stderr, "  -D #             thickness of slide \n");
	fprintf(stderr, "  -e #             error tolerance (default 0.0001) \n");
	fprintf(stderr, "  -F #             use this scattering coefficient \n");
	fprintf(stderr, "  -g #             scattering anisotropy (default 0) \n");
	fprintf(stderr, "  -G #             't' (one top) or 'b' (one bottom) slide\n");
	fprintf(stderr, "  -h               display help\n");
	fprintf(stderr, "  -M #             number of Monte Carlo iterations\n");
	fprintf(stderr, "  -n #             specify index of refraction of slab\n");
	fprintf(stderr, "  -N #             specify index of refraction of slides\n");
	fprintf(stderr, "  -o filename      explicitly specify filename for output\n");
	fprintf(stderr, "  -p #             # of Monte Carlo photons (default 100000)\n");
	fprintf(stderr, "                   a negative number is max MC time in milliseconds\n");
	fprintf(stderr, "  -q #             number of quadrature points (default=8)\n");
	fprintf(stderr, "  -r #             total reflection measurement\n");
	fprintf(stderr, "  -s               show sphere and MC light loss effects\n");
	fprintf(stderr, "  -S #             number of spheres used\n");
	fprintf(stderr, "  -t #             total transmission measurement\n");
	fprintf(stderr, "  -u #             unscattered transmission measurement\n");
	fprintf(stderr, "  -v               version information\n");
	fprintf(stderr, "  -V 0             verbosity low --- no output to stderr\n");
	fprintf(stderr, "  -V 1             verbosity moderate \n");
	fprintf(stderr, "  -V 2             verbosity high\n");
	fprintf(stderr, "  -x #             set debugging level\n");
	fprintf(stderr, "Examples:\n");
	fprintf(stderr, "  iad data                    optical values put in data.txt\n");
	fprintf(stderr, "  iad -c 1 data               assume M_R has no unscattered reflectance\n");
	fprintf(stderr, "  iad -c 2 data               assume M_T has no unscattered transmittance\n");
	fprintf(stderr, "  iad -c 3 data               assume M_R & M_T have no unscattered light\n");
	fprintf(stderr, "  iad -e 0.0001 data          better convergence to R & T values\n");
	fprintf(stderr, "  iad -o out data             calculated values in out\n");
	fprintf(stderr, "  iad -r 0.3                  R_total=0.3, b=inf, find albedo\n");
	fprintf(stderr, "  iad -r 0.3 -t 0.4           R_total=0.3, T_total=0.4, find a,b,g\n");
	fprintf(stderr, "  iad -r 0.3 -t 0.4 -n 1.5    R_total=0.3, T_total=0.4, n=1.5, find a,b\n");
	fprintf(stderr, "  iad -r 0.3 -t 0.4           R_total=0.3, T_total=0.4, find a,b\n");
	fprintf(stderr, "  iad -p 1000 data            only 1000 photons\n");
	fprintf(stderr, "  iad -p -100 data            allow only 100ms per iteration\n");
	fprintf(stderr, "  iad -q 4 data               four quadrature points\n");
	fprintf(stderr, "  iad -M 0 data               no MC    (iad)\n");
	fprintf(stderr, "  iad -M 1 data               MC once  (iad -> MC -> iad)\n");
	fprintf(stderr, "  iad -M 2 data               MC twice (iad -> MC -> iad -> MC -> iad)\n");
	fprintf(stderr, "  iad -M 0 -q 4 data          Fast and crude conversion\n");
	fprintf(stderr, "  iad -s data                 show sphere and MC light loss effects\n");
	fprintf(stderr, "  iad -G t data               one top slide with properties from data.rxt\n");
	fprintf(stderr, "  iad -G b -N 1.5 -D 1 data   use 1 bottom slide with n=1.5 and thickness=1\n");
	fprintf(stderr, "  iad -x   1 data             show sphere and MC effects\n");
	fprintf(stderr, "  iad -x   2 data             DEBUG_GRID\n");
	fprintf(stderr, "  iad -x   4 data             DEBUG_ITERATIONS\n");
	fprintf(stderr, "  iad -x   8 data             DEBUG_LOST_LIGHT\n");
	fprintf(stderr, "  iad -x  16 data             DEBUG_SPHERE_EFFECTS\n");
	fprintf(stderr, "  iad -x  32 data             DEBUG_BEST_GUESS\n");
	fprintf(stderr, "  iad -x  64 data             DEBUG_EVERY_CALC\n");
	fprintf(stderr, "  iad -x 128 data             DEBUG_SEARCH\n");
	fprintf(stderr, "  iad -x 255 data             all debugging output\n\n");
	fprintf(stderr, "  apply iad data1 data2       process multiple files\n\n");
	fprintf(stderr, "Report bugs to <prahl@@bme.ogi.edu>\n\n");
	exit(0);
}

@ This can only be called immediately after |Invert_RT|
You have been warned!
@<calculate coefficients function@>=
static void calculate_coefficients(struct measure_type m,
                            struct invert_type r,
                            double *LR, double *LT, double *musp, double *mua)
{
	double delta;
	*LR = 0;
	*LT = 0;

	Calculate_Distance(LR, LT, &delta);

    if (r.default_mus != UNINITIALIZED) {
		*musp = r.default_mus * (1.0-r.g);
		*mua  = r.default_mus * (1.0-r.a)/r.a;
		return;
	}
	
	if (r.default_mua != UNINITIALIZED) {
		*mua  = r.default_mua;
		*musp = r.default_mua * r.a/(1.0-r.a) * (1-r.g);
		return;
	}			

	if (r.default_b == HUGE_VAL || r.b == HUGE_VAL) {
		if (r.a == 0) {
			*musp = 0.0;
			*mua  = 1.0;
			return;
		}
		*musp = 1.0-r.g;
		*mua  = (1.0-r.a)/r.a;
		return;
	}

	*musp  = r.a*r.b/m.slab_thickness*(1.0-r.g);
	*mua   = (1-r.a)*r.b/m.slab_thickness;
}

@ @<print error legend@>=
static void print_error_legend(void)
{
	fprintf(stderr, "----------------- Sorry, but ... mistakes were made ---------------\n");
	fprintf(stderr, "  0-9 ==> Monte Carlo Iteration\n");
	fprintf(stderr, "   *  ==> Successful Calculation\n");
	fprintf(stderr, "-------------------------------------------------------------------\n");
	fprintf(stderr, "   R  ==> Bad Reflection\n");
	fprintf(stderr, "   T  ==> Bad Transmission\n");
	fprintf(stderr, "   U  ==> Bad Unscattered Transmission\n");
	fprintf(stderr, "   L  ==> Reflection + Transmission +  Monte Carlo Light Losses > 1\n");
	fprintf(stderr, "   !  ==> Reflection + Transmission + Unscattered > 1\n");
	fprintf(stderr, "   +  ==> Too many adding-doubling iterations\n\n");
}


@ returns a new string consisting of s+t
@<stringdup together function@>=
static char *  strdup_together(char *s, char *t)
{
    char * both;
    
    if (s==NULL) {
        if (t==NULL) return NULL;
        return strdup(t);
    }
    
    if (t==NULL)
        return strdup(s);
        
    both = malloc(strlen(s) + strlen(t) + 1);
    if (both == NULL)
        fprintf(stderr, "Could not allocate memory for both strings.\n");

    strcpy(both, s);
    strcat(both, t);
    return both;
}

@ assume that start time has already been set
@<seconds elapsed function@>=

static double seconds_elapsed(clock_t start_time)
{   
    clock_t finish_time = clock();
    return (double)(finish_time-start_time)/CLOCKS_PER_SEC;
}

@ given a string and an array, this fills the array with numbers
from the string.  The numbers should be separated by spaces.

Returns 0 upon successfully filling |n| entries, returns
1 for any error.

@<parse string into array function@>=

static int parse_string_into_array(char *s, double *a, int n)
{   
    char *t, *last, *r;
    int i=0;
    t = s;  
    last = s + strlen(s);
       
    while(t<last) {

    	/* a space should mark the end of number */
    	r = t;
    	while (*r != ' ' && *r != '\0') r++;
    	*r = '\0';
    	
    	/* parse the number and save it */
    	if (sscanf(t, "%lf", &(a[i]) )==0) return 1;    	
    	i++;    	

		/* are we done ? */
    	if (i==n) return 0;
    	
    	/* move pointer just after last number */
    	t=r+1;
    }
    return 1;
}

@ @<print dot function@>=

static void print_dot(clock_t start_time, int err, int count, int points,
                       int final, int *any_error)
{
	static int counter = 1;
	
	if (err == IAD_NO_ERROR) {
		if (final==99)
			fprintf(stderr,"*");
		else
			fprintf(stderr,"%d", (int) fmod(final,10));
	} else {
		*any_error = 1;
		if (err == IAD_TOO_MANY_ITERATIONS)
			fprintf(stderr,"+");
			
		else if (err == IAD_R_LT_ZERO      ||
		         err == IAD_R_GT_ONE       ||
		         err == IAD_RT_GT_ONE      ||
		         err == IAD_RT_LT_MINIMUM  ||
		         err == IAD_RD_LT_ZERO      )
			fprintf(stderr,"R");
		         
		else if (err == IAD_T_LT_ZERO  ||
		         err == IAD_T_GT_ONE   ||
		         err == IAD_RD_IS_ZERO_BUT_NOT_TD)
			fprintf(stderr,"T");
		         
		else if (err == IAD_TU_LT_ZERO || err == IAD_TU_GT_ONE)
			fprintf(stderr,"U");
		         
		else if (err == IAD_R_PLUS_T_GT_ONE    ||
		         err == IAD_RT_PLUS_TT_GT_ONE  ||
		         err == IAD_R_PLUS_T_PLUS_TU_GT_ONE)
			fprintf(stderr,"!");

		else if (err == IAD_EXCESSIVE_LIGHT_LOSS) 
			fprintf(stderr,"L");
			
		else {
			if (err >= IAD_AS_NOT_VALID && err <= IAD_TOO_MANY_LAYERS) 
				fprintf(stderr,"\nBad sphere parameter (error=%d)\n",err);
			else if (err == IAD_MEMORY_ERROR)
				fprintf(stderr,"\nMemory error\n");
			else if (err == IAD_FILE_ERROR) 
				fprintf(stderr,"\nFile error\n");
			exit(1);
		}		
	}
		   
	if (counter % 50 == 0) {
		double rate = (seconds_elapsed(start_time) / points);
		fprintf(stderr, "  %3d done (%5.2f s/pt)\n", points,rate); 
	} else if (counter % 10 == 0)
		fprintf(stderr," ");
		
    counter++;
	fflush(stderr);  /* Needed for windows */
}

