@*1 IAD Public.

This contains the routine |Inverse_RT| that should generally be
the basic entry point into this whole mess.  Call this routine with
the proper values and true happiness is bound to be yours.

Altered accuracy of the
standard method of root finding from 0.001 to 0.00001.
Note, it really doesn't help to change the method from
|ABSOLUTE| to |RELATIVE|, but I did anyway.  (3/3/95)

@(iad_pub.c@>=
#include <stdio.h>
#include <math.h>
#include "nr_util.h"
#include "ad_globl.h"
#include "ad_frsnl.h"
#include "iad_type.h"
#include "iad_util.h"
#include "iad_find.h"
#include "iad_pub.h"
#include "iad_io.h"
#include "mc_lost.h"

@<Definition for |Inverse_RT|@>@;
	@<Definition for |measure_OK|@>@;
	@<Definition for |determine_search|@>@;
	@<Definition for |Initialize_Result|@>@;
    @<Definition for |Initialize_Measure|@>@;
	@<Definition for |ez_Inverse_RT|@>@;
    @<Definition for |Spheres_Inverse_RT|@>@;
	
@ All the information that needs to be written to
the header file \.{iad\_pub.h}.  This eliminates the need to maintain a set of 
header files as well.

@(iad_pub.h@>=
	@<Prototype for |Inverse_RT|@>;
	@<Prototype for |measure_OK|@>;
	@<Prototype for |determine_search|@>;
	@<Prototype for |Initialize_Result|@>;
	@<Prototype for |ez_Inverse_RT|@>;
    @<Prototype for |Initialize_Measure|@>;

@ Here is the header file needed to access one interesting routine in the 
\.{libiad.so} library.

@(lib_iad.h@>=
	@<Prototype for |ez_Inverse_RT|@>;
	@<Prototype for |Spheres_Inverse_RT|@>;

@*2 Inverse RT.
|Inverse_RT| is the main function in this whole package.
You pass the variable |m| containing
your experimentally measured values to the function
|Inverse_RT|.  It hopefully returns the optical properties in |r| 
that are appropriate for your experiment.  

history:
6/8/94 changed the way the program writes error stuff.  Use stderr 
uniformly throughout.

@<Prototype for |Inverse_RT|@>=
	void Inverse_RT(struct measure_type m, struct invert_type *r)

@ @<Definition for |Inverse_RT|@>=
	@<Prototype for |Inverse_RT|@>
{
	r->found = FALSE;
	@<Exit with bad input data@>@;
	
    r->search = determine_search(m,*r);
    
	@<Find the optical properties@>@;
	if ( r->error ==IAD_NO_ERROR && r->final_distance <= r->tolerance) r->found=TRUE;
}

@ There is no sense going to all the trouble to try a multivariable
minimization if the input data is bogus.  So I wrote a 
single routine |measure_OK| to do just this.

@<Exit with bad input data@>=
	r->error=measure_OK(m,*r);
	if (r->method.quad_pts<4) 
		r->error = IAD_QUAD_PTS_NOT_VALID;
		
	if (0 && (r->error != IAD_NO_ERROR)) 
		return;
  
@ Now I fob the real work off to the unconstrained minimization
routines.  Ultimately, I would like to replace all these by constrained
minimization routines.  Actually the first five already are constrained.
The real work will be improving the last five because these are 2-D
minimization routines.

@<Find the optical properties@>=
switch (r->search){
	case FIND_A:  U_Find_A(m,r);
			      break;
	case FIND_B:  U_Find_B(m,r);
			      break;
	case FIND_G:  U_Find_G(m,r);
			      break;
	case FIND_Ba: U_Find_Ba(m,r);
			      break;
	case FIND_Bs: U_Find_Bs(m,r);
			      break;

	case FIND_AB: U_Find_AB(m,r);
			      break;
	case FIND_AG: U_Find_AG(m,r);
			      break;
	case FIND_BG: U_Find_BG(m,r);
			      break;
	case FIND_BsG: U_Find_BsG(m,r);
			      break;
	case FIND_BaG: U_Find_BaG(m,r);
			      break;
}
if (r->iterations==IAD_MAX_ITERATIONS)
	r->error=IAD_TOO_MANY_ITERATIONS;

@*2 Validation.

@ Now the question is --- just what is bad data?  Here's the prototype.

@<Prototype for |measure_OK|@>=
int measure_OK(struct measure_type m, struct invert_type r)

@ It would just be nice to stop computing with bad data.  This does not
work in practice becasue it turns out that there is often bogus data in a full wavelength
scan.  Often the reflectance is too low for short wavelengths and at
long wavelengths the detector (photomultiplier tube) does not work worth
a damn.

More recently, constraining the albedo or scattering or absorption
coefficient has become common.  

@<Definition for |measure_OK|@>=
@<Prototype for |measure_OK|@>
{
	double rt, tt, rd, rc, td, tc;
        
	int error=IAD_NO_ERROR;

	@<Test easy cases@>@;
	@<Calculate specular limits for reflection and transmission@>@;
	@<Check specular limits@>@;
	@<Check sphere parameters@>@;
	return error;
}

@ The easy cases consist of making sure that the reflection and
transmission and their sum is between zero and one --- without
worrying overmuch about the specular reflection.  We begin by
making sure that the reflection is valid.

Note that I just fake the case for corrections to $R+T>1$ when spheres are
present ... it is possible for $R+T>1$ by a bit in this case! 


@<Test easy cases@>=
  
    if (m.m_r < 0)
        error = IAD_R_LT_ZERO;
    if (m.m_r > 1)
        error = IAD_R_GT_ONE;
	
@ We only should check the transmission if it is present.  This
means that the number of measurements must be 2 or 3.  We can
also make a quick check to ensure that energy conservation is 
fulfilled.

@<Test easy cases@>=
    if (m.num_measures > 1) {
        if (m.m_t < 0)
            error = IAD_T_LT_ZERO;
        if (m.m_t > 1)
            error = IAD_T_GT_ONE;
        if (r.default_a && m.m_t+m.m_r > 1)
            error = IAD_R_PLUS_T_GT_ONE;      
    }
  
@ Finally the unscattered transmission value should be checked if
it is present.  If the specular reflection is not included in the
transmission value, then another check on energy conservation may
be made.

@<Test easy cases@>=
    if (m.num_measures == 3) {
        if (m.m_u > 1)
            error = IAD_TU_GT_ONE;
        if (m.m_u <0 )
            error = IAD_TU_LT_ZERO;
        if (!m.sphere_with_tc && m.m_r+m.m_t+m.m_u>1.0)
            error = IAD_R_PLUS_T_PLUS_TU_GT_ONE;
  }


@ Now we make estimates for the specular reflection and
transmission so that the diffuse reflection and transmission
for the sample can be guessed.

@<Calculate specular limits for reflection and transmission@>=
 Estimate_RT(m, r.slab, &rt, &tt, &rd, &rc, &td, &tc);

@ The only cases that remain are those values that have been
possibly altered by the addition or subtraction of the unscattered
reflection or transmission.  These are the diffuse reflection
(it could have become negative), the total reflection (it could
have become greater than one), the diffuse transmission
(it could have become negative), the total transmission (it could
have become greater than one) and the sum of the total transmission
and total reflection.  These are more stringent tests than those
above, but are very useful for skipping bogus data points.

@<Check specular limits@>=
  if (r.default_a && rd < 0)
    error = IAD_RD_LT_ZERO;
  if (rt > 1.0)
    error = IAD_RT_GT_ONE;
  if (td < 0)
    error = IAD_TD_LT_ZERO;
  if (tt > 1.0)
    error = IAD_TT_GT_ONE;
  if (r.default_a && tt + rt > 1.0)
    error = IAD_RT_PLUS_TT_GT_ONE;

@ Another specular limit has arisen.  Namely the non-scattering case.
This only can be checked if there are two or three measurements.

There is a definite bound on the minimum reflectance from a sample.
If you have a sample with a given transmittance |m_t|, the minimum
reflectance possible is found by assuming that the sample does not scatter any light. 

Knowledge of the indicies of refraction makes it a relatively simple matter to
determine the optical thickness |b=mu_a*d| ofthe slab. The minimum reflection
is obtained by including all the specular reflectances from all the surfaces.

@<Check specular limits@>=
if (m.num_measures == 3 
    && r.search != FIND_A 
    && r.search != FIND_B 
    && r.search != FIND_G
    && m.m_u > 0) {
	double rmin, tmin, b;
	
	b = What_Is_B(r.slab, m.m_u);
	
	Sp_mu_RT(r.slab.n_top_slide, r.slab.n_slab, r.slab.n_bottom_slide, 
			 r.slab.b_top_slide, b,             r.slab.b_bottom_slide, 1.0, &rmin, &tmin);

	if (m.m_r + 0.0001 < rmin) {
	error = IAD_RT_LT_MINIMUM;
	fprintf(stderr, "R = %7.5f  T = %7.5f but Min R= %7.5f\n", m.m_r, m.m_t, rmin);
	}
}

	
@ There is one more point that needs to be checked.  If the diffuse
reflection is zero, then the diffuse transmission must also be zero
(because the albedo must be zero).  Note that the converse is not true
since there may just be insufficient light to detect on the far side
of the sample.

I needed to disable this check for when searching only for the scattering
or the absorption coefficient alone.

@<Check specular limits@>=
	if (rd==0 && td!=0 && r.default_a)
		if (r.search!=FIND_Ba && r.search!=FIND_Bs) 
			error = IAD_RD_IS_ZERO_BUT_NOT_TD;
	
@ Make sure that reflection sphere parameters are reasonable

@<Check sphere parameters@>=
  
  
  if (m.num_spheres != 0) {
  
  	if (m.as_r < 0 || m.as_r >= 0.2) 
  		error=IAD_AS_NOT_VALID;
  		
  	if (m.ad_r < 0 || m.ad_r >= 0.2) 
  		error=IAD_AD_NOT_VALID;
  	
  	if (m.ae_r < 0 || m.ae_r >= 0.2) 
  		error=IAD_AE_NOT_VALID;

  	if (m.rw_r < 0 || m.rw_r > 1.0) 
  		error=IAD_RW_NOT_VALID;

  	if (m.rd_r < 0 || m.rd_r > 1.0) 
  		error=IAD_RD_NOT_VALID;

  	if (m.rstd_r < 0 || m.rstd_r > 1.0) 
  		error=IAD_RSTD_NOT_VALID;

  	if (m.f_r < 0 || m.f_r > 1) 
  		error=IAD_F_NOT_VALID;
  }
  
@ Make sure that transmission sphere parameters are reasonable

@<Check sphere parameters@>=
  
  if (m.num_spheres != 0) {
  
  	if (m.as_t < 0 || m.as_t >= 0.2) 
  		error=IAD_AS_NOT_VALID;
  		
  	if (m.ad_t < 0 || m.ad_t >= 0.2) 
  		error=IAD_AD_NOT_VALID;
  	
  	if (m.ae_t < 0 || m.ae_t >= 0.2) 
  		error=IAD_AE_NOT_VALID;

  	if (m.rw_t < 0 || m.rw_r > 1.0) 
  		error=IAD_RW_NOT_VALID;

  	if (m.rd_t < 0 || m.rd_t > 1.0) 
  		error=IAD_RD_NOT_VALID;

  	if (m.rstd_t < 0 || m.rstd_t > 1.0) 
  		error=IAD_RSTD_NOT_VALID;

  	if (m.f_t < 0 || m.f_t > 1) 
  		error=IAD_F_NOT_VALID;
  }

@*2 Searching Method.  

The original idea was that this routine would automatically determine
what optical parameters could be figured out from the input data.  This
worked fine for a long while, but I discovered that often it was convenient
to constrain the optical properties in various ways.  Consequently, this
routine got more and more complicated.  

What should be done is to figure out whether the search will be 1D or 2D
and split this routine into two parts.

It would be nice to enable the user to constrain two parameters, but
the infrastructure is missing at this point.

@<Prototype for |determine_search|@>=
search_type determine_search(struct measure_type m, struct invert_type r)

@ This routine is responsible for selecting the appropriate 
optical properties to determine.  

@<Definition for |determine_search|@>=
@<Prototype for |determine_search|@>
{
	double rt, tt, rd, td, tc, rc;
	int search=0;
	int independent = m.num_measures;

	Estimate_RT(m, r.slab, &rt, &tt, &rd, &rc, &td, &tc);

	if (tc==0 && independent == 3) /* no information in tc */
		independent--;

	if (rd==0 && independent == 2) /* no information in rd */
		independent--;
		
	if (td==0 && independent == 2) /* no information in td */
		independent--;

	if (independent == 1) {
		@<One parameter search@>@;
	}
	
	else if (independent == 2) {
		@<Two parameter search@>@;
	}

	/* three real parameters with information! */	
	else {
		search = FIND_AG;
	}
	
	if (Debug(DEBUG_SEARCH)) {
		fprintf(stderr,"independent measurements = %3d\n",independent);
		if (search==FIND_A)    fprintf(stderr,"search = FIND_A\n");
		if (search==FIND_B)    fprintf(stderr,"search = FIND_B\n");
		if (search==FIND_AB)   fprintf(stderr,"search = FIND_AB\n");
		if (search==FIND_AG)   fprintf(stderr,"search = FIND_AG\n");
		if (search==FIND_AUTO) fprintf(stderr,"search = FIND_AUTO\n");
		if (search==FIND_BG)   fprintf(stderr,"search = FIND_BG\n");
		if (search==FIND_BaG)  fprintf(stderr,"search = FIND_BaG\n");
		if (search==FIND_BsG)  fprintf(stderr,"search = FIND_BsG\n");
		if (search==FIND_Ba)   fprintf(stderr,"search = FIND_Ba\n");
		if (search==FIND_Bs)   fprintf(stderr,"search = FIND_Bs\n");
		if (search==FIND_G)    fprintf(stderr,"search = FIND_G\n");
	}
	
	return search;
}


@ The fastest inverse problems are those in which just one
measurement is known.  This corresponds to a simple one-dimensional
minimization problem.  The only complexity is deciding exactly what
should be allowed to vary.  The basic assumption is that the 
anisotropy has been specified or will be assumed to be zero.

If the anistropy is assumed known, then one other assumption will
allow us to figure out the last parameter to solve for.

Ultimately, if no default values are given, then we look at the
value of the diffuse reflectance.  If this is zero, then we solve
for the optical thickness, otherwise the sample is assumed infinitely
thick and the albedo is found.

@<One parameter search@>=
	if (r.default_a  != UNINITIALIZED) {
		if (r.default_a  == 0)
			search =  FIND_B;
		else {
			if (td == 0)
				search = FIND_G;
			else
				search = FIND_B;
		}
	}
	
	else if (r.default_b  != UNINITIALIZED)
     	search = FIND_A;
     	
	else if (r.default_bs != UNINITIALIZED)
     	search = FIND_Ba;
     	
	else if (r.default_ba != UNINITIALIZED)
     	search = FIND_Bs;
     
    else if (rd != 0 || td == 0)
    	search = FIND_A;
    else
    	search = FIND_B;


@ If the absorption depth $\mu_a d$ is constrained return |FIND_BsG|.
Recall that I use the bizarre mnemonic $bs=\mu_s d$ here and so this
means that the program will search over various values of $\mu_s d$
and $g$.

If there are just two measurements then I assume that the 
anisotropy is not of interest and the only thing to calculate
is the reduced albedo and optical thickness based on an assumed
anisotropy.  

@<Two parameter search@>=
	if (r.default_a != UNINITIALIZED) {
		
		if ((r.default_a  == 0) || (r.default_g  != UNINITIALIZED))
			search =  FIND_B;
		else
			search =  FIND_BG;
	
	} else if (r.default_b  != UNINITIALIZED) {
	
		if (r.default_g  != UNINITIALIZED)
			search =  FIND_A;
		else
			search =  FIND_AG;

	} else if (r.default_ba != UNINITIALIZED) {
	
		if (r.default_g  != UNINITIALIZED)
			search =  FIND_Bs;
		else
			search =  FIND_BsG;	

	} else if (r.default_bs != UNINITIALIZED) {
	
		if (r.default_g  != UNINITIALIZED)
			search =  FIND_Ba;
		else
			search =  FIND_BaG;	
	
	} else
		search = FIND_AB;

@ This little routine just stuffs reasonable values into the 
structure we use to return the solution.  This does not replace 
the values for |r.default_g| nor for |r.method.quad_pts|.  Presumably
these have been set correctly elsewhere.

@<Prototype for |Initialize_Result|@>=
void Initialize_Result(struct measure_type m, struct invert_type *r)

@ @<Definition for |Initialize_Result|@>=
	@<Prototype for |Initialize_Result|@>
{
@<Fill |r| with reasonable values@>@;
}

@ Start with the optical properties.
@<Fill |r| with reasonable values@>=
    r->a = 0.0;
    r->b = 0.0;
    r->g = 0.0;

@ Continue with other useful stuff.
@<Fill |r| with reasonable values@>=
    r->found = FALSE;
	r->tolerance = 0.0001;
	r->MC_tolerance = 0.01;  /* percent */
	r->search = FIND_AUTO;
	r->metric = RELATIVE;
	r->final_distance = 10;
	r->iterations =0;
	r->error = IAD_NO_ERROR;

@ The defaults might be handy

@<Fill |r| with reasonable values@>=
	r->default_a=UNINITIALIZED;
	r->default_b=UNINITIALIZED;
	r->default_g=UNINITIALIZED;
	r->default_ba=UNINITIALIZED;
	r->default_bs=UNINITIALIZED;
	r->default_mua=UNINITIALIZED;
	r->default_mus=UNINITIALIZED;


@ It is necessary to set up the slab correctly so, I stuff reasonable
values into this record as well.
@<Fill |r| with reasonable values@>=
	
	r->slab.a = 0.5;
	r->slab.b = 1.0;
	r->slab.g = 0;
	r->slab.phase_function = HENYEY_GREENSTEIN;
	r->slab.n_slab = m.slab_index;
	r->slab.n_top_slide = m.slab_top_slide_index;
	r->slab.n_bottom_slide = m.slab_bottom_slide_index;
	r->slab.b_top_slide = m.slab_top_slide_b;
	r->slab.b_bottom_slide = m.slab_bottom_slide_b;
	
	r->method.a_calc=0.5;
	r->method.b_calc=1;
	r->method.g_calc=0.5;
	r->method.quad_pts=8;
	r->method.b_thinnest = 1.0/32.0;

@*2 EZ Inverse RT.
|ez_Inverse_RT| is a simple interface to the main function |Inverse_RT|
in this package.  It eliminates the need for complicated data structures
so that the command line interface (as well as those to Perl and Mathematica)
will be simpler.  This function assumes
that the reflection and transmission include specular reflection and that
the transmission also include unscattered transmission.

Other assumptions are that the top and bottom slides have the same
index of refraction, that the illumination is collimated.  Of course
no sphere parameters are included.

@<Prototype for |ez_Inverse_RT|@>=
	void ez_Inverse_RT(double n, double nslide, double UR1, double UT1, double Tc, 
	                   double *a, double *b, double *g, int *error)

@ @<Definition for |ez_Inverse_RT|@>=
	@<Prototype for |ez_Inverse_RT|@>
{
  struct measure_type m;
  struct invert_type r;
  *a = 0;
  *b = 0;
  *g = 0;

  Initialize_Measure(&m);
  
  m.slab_index = n;
  m.slab_top_slide_index=nslide;
  m.slab_bottom_slide_index=nslide;
  
  m.num_measures=3;
  fprintf(stderr,"ut1=%f\n",UT1);
  fprintf(stderr,"Tc=%f\n",Tc);
  if (UT1 == 0) m.num_measures--;
  if (Tc  == 0) m.num_measures--;

  m.m_r = UR1;
  m.m_t = UT1;
  m.m_u = Tc;

  Initialize_Result(m,&r);
  r.method.quad_pts=8;
  
  Inverse_RT (m, &r);

  *error = r.error;
  if (r.error == IAD_NO_ERROR) {
  	*a = r.a;
  	*b = r.b;
  	*g = r.g;
  } 
}

@ @<Prototype for |Initialize_Measure|@>=
void Initialize_Measure(struct measure_type *m)

@ @<Definition for |Initialize_Measure|@>=
	@<Prototype for |Initialize_Measure|@>
{
        double default_sphere_d   = 8.0 * 25.4;
        double default_sample_d   = 0.5 * 25.4;
        double default_detector_d = 0.1 * 25.4;
        double default_entrance_d = 0.5 * 25.4;
        double sphere = default_sphere_d * default_sphere_d;
        
        m->slab_index=1.0;
        m->slab_top_slide_index=1.0;
        m->slab_top_slide_b=0.0;
        m->slab_top_slide_thickness=0.0;
        m->slab_bottom_slide_index=1.0;
        m->slab_bottom_slide_b=0.0;
        m->slab_bottom_slide_thickness=0.0;
        m->slab_thickness=1.0;
        
        m->num_spheres=0;
        m->num_measures=1;
        
        m->sphere_with_rc=1.0;
        m->sphere_with_tc=1.0;
        
        m->m_r=0.0;
        m->m_t=0.0;
        m->m_u=0.0;
        
	    m->d_sphere_r = default_sphere_d;
		m->as_r = default_sample_d * default_sample_d / sphere;
		m->ad_r = default_detector_d * default_detector_d / sphere;
		m->ae_r = default_entrance_d * default_entrance_d / sphere;
		m->aw_r = 1.0 - m->as_r - m->ad_r -m->ae_r;
		m->rd_r = 0.0;
		m->rw_r = 1.0;
		m->rstd_r = 1.0;
		m->f_r = 0.0;
		
	    m->d_sphere_t = default_sphere_d;
		m->as_t = m->as_r;
		m->ad_t = m->ad_r;
		m->ae_t = m->ae_r;
		m->aw_t = m->aw_r;
		m->rd_t = 0.0;
		m->rw_t = 1.0;
		m->rstd_t = 1.0;
		m->f_t = 0.0;

	    m->lambda = 0.0;
	    m->d_beam = 0.0;
	    m->ur1_lost = 0;
	    m->uru_lost = 0;
	    m->ut1_lost = 0;
	    m->utu_lost = 0;
	    
}

@ To avoid interfacing with C-structures it is necessary to pass the information
as arrays.  Here I have divided the experiment into (1) setup, (2) reflection 
sphere coefficients, (3) transmission sphere coefficients, (4) measurements, and (5) 
results.

@<Prototype for |Spheres_Inverse_RT|@>=
	void Spheres_Inverse_RT(double *setup, 
	                        double *analysis, 
	                        double *sphere_r, 
	                        double *sphere_t,
	                        double *measurements,
	                        double *results)
	
@ @<Definition for |Spheres_Inverse_RT|@>=
	@<Prototype for |Spheres_Inverse_RT|@>
{
	struct measure_type m;
	struct invert_type r;
	long num_photons;
    double ur1,ut1,uru,utu;
    int i, mc_runs = 1;
	
	Initialize_Measure(&m);
	
	@<handle setup @>@;
	@<handle reflection sphere @>@;
	@<handle transmission sphere @>@;
	@<handle measurement @>@;

	Initialize_Result(m,&r);
	results[0]=0;
	results[1]=0;
	results[2]=0;

	@<handle analysis @>@;
	
	Inverse_RT (m, &r);
	for (i=0; i<mc_runs; i++) {
		MC_Lost(m, r, num_photons, &ur1, &ut1, &uru, &utu, 
					 &m.ur1_lost, &m.ut1_lost, &m.uru_lost, &m.utu_lost);   
		Inverse_RT (m, &r);
	}
	
	if (r.error == IAD_NO_ERROR) {
		results[0]=(1-r.a)*r.b/m.slab_thickness;
		results[1]=(r.a  )*r.b/m.slab_thickness;
		results[2]=r.g;
	} 
	
	results[3]=r.error;
}

@ These are in exactly the same order as the parameters in the .rxt header
@<handle setup @>=
{  
	double d_sample_r, d_entrance_r, d_detector_r;
	double d_sample_t, d_entrance_t, d_detector_t;
	
	m.slab_index               = setup[0];
	m.slab_top_slide_index     = setup[1];
	m.slab_thickness           = setup[2];
	m.slab_top_slide_thickness = setup[3];
	m.d_beam                   = setup[4];
	m.rstd_r                   = setup[5];
	m.num_spheres              = (int) setup[6];  
	
	m.d_sphere_r               = setup[7];
	d_sample_r                 = setup[8];
	d_entrance_r               = setup[9];
	d_detector_r               = setup[10];
	m.rw_r                     = setup[11];
	
	m.d_sphere_t               = setup[12];
	d_sample_t                 = setup[13];
	d_entrance_t               = setup[14];
	d_detector_t               = setup[15];
	m.rw_t                     = setup[16];
	
	r.default_g                = setup[17];
	num_photons                = (long) setup[18];
	
	m.as_r = (d_sample_r   / m.d_sphere_r) * (d_sample_r   / m.d_sphere_r);
	m.ae_r = (d_entrance_r / m.d_sphere_r) * (d_entrance_r / m.d_sphere_r);
	m.ad_r = (d_detector_r / m.d_sphere_r) * (d_detector_r / m.d_sphere_r);
	m.aw_r = 1.0 - m.as_r - m.ae_r - m.ad_r;
	m.as_t = (d_sample_t   / m.d_sphere_t) * (d_sample_t   / m.d_sphere_t);
	m.ae_t = (d_entrance_t / m.d_sphere_t) * (d_entrance_t / m.d_sphere_t);
	m.ad_t = (d_detector_t / m.d_sphere_t) * (d_detector_t / m.d_sphere_t);
	m.aw_t = 1.0 - m.as_t - m.ae_t - m.ad_t;

	m.slab_bottom_slide_index = m.slab_top_slide_index;
    m.slab_bottom_slide_thickness = m.slab_top_slide_thickness;
}

@ @<handle analysis@>=

  r.method.quad_pts        = (int) analysis[0];
  mc_runs                  = (int) analysis[1];
  
@  @<handle measurement @>=
  m.m_r = measurements[0];
  m.m_t = measurements[1];
  m.m_u = measurements[2];
 
  m.num_measures=3;
  fprintf(stderr,"m.m_t=%f\n",m.m_t);
  fprintf(stderr,"m.m_u=%f\n",m.m_u);
  if (m.m_t == 0) m.num_measures--;
  if (m.m_u == 0) m.num_measures--;

@  @<handle reflection sphere @>=
  m.as_r    = sphere_r[0];
  m.ae_r    = sphere_r[1];
  m.ad_r    = sphere_r[2];
  m.rw_r    = sphere_r[3];
  m.rd_r    = sphere_r[4];
  m.rstd_r  = sphere_r[5];
  m.f_r     = sphere_r[7];
  
@  @<handle transmission sphere @>=
  m.as_t    = sphere_t[0];
  m.ae_t    = sphere_t[1];
  m.ad_t    = sphere_t[2];
  m.rw_t    = sphere_t[3];
  m.rd_t    = sphere_t[4];
  m.rstd_t  = sphere_t[5];
  m.f_t     = sphere_t[7];

